﻿using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.BusinessLogicsContracts;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels.Enums;
using Microsoft.Extensions.Logging;

namespace SoftwareInstallationBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }
        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                throw new ArgumentException(
                    $"Статус заказа должен быть {OrderStatus.Неизвестен}", nameof(model));
            }
            model.Status = OrderStatus.Принят;
            model.DateCreate = DateTime.Now;
            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            model.DateImplement = DateTime.Now;
            return SetOrderStatus(model, OrderStatus.Выдан);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return SetOrderStatus(model, OrderStatus.Готов);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderName.Id:{ Id} ", model?.Id);
            var list = (model == null) ? _orderStorage.GetFullList() :
            _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return SetOrderStatus(model, OrderStatus.Выполняется);
        }
        private bool CheckModel(OrderBindingModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentException("Количество изделий в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentException("Суммарная стоимость заказа должна быть больше 0", nameof(model.Sum));
            }
            if (model.DateCreate > model.DateImplement)
            {
                throw new ArgumentException("Время создания заказа не может быть больше времени его выполнения", nameof(model.DateImplement));
            }
            return true;
        }
        private bool SetOrderStatus(OrderBindingModel model, OrderStatus orderStatus)
        {
            var viewModel = _orderStorage.GetElement(new() { Id = model.Id });
            if (viewModel == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if ((int)viewModel.Status + 1 != (int)orderStatus)
            {
                throw new ArgumentException($"Попытка перевести заказ не в следующий статус: " +
                    $"Текущий статус: {viewModel.Status} \n" +
                    $"Планируемый статус: {orderStatus} \n" +
                    $"Доступный статус: {(OrderStatus)((int)viewModel.Status + 1)}",
                    nameof(viewModel));
            }
            model.Status = orderStatus;
            model.Sum = viewModel.Sum;
            model.Count = viewModel.Count;
            model.DateCreate = viewModel.DateCreate;
            model.PackageId = viewModel.PackageId;
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Ошибка операции обновления");
                return false;
            }
            return true;
        }
    }
}